% PStill run-time script, F. Siegert, Februar 2000, v1.17
% Use this to do specific additions to the PDF annotation engine
% DO NOT TOUCH IT IF YOU DO NOT UNDERSTAND IT EXCEPT FOR THE LINE
% MARKED **** IN CASE OF TROUBLES WITH ANNOTATIONS

variable annotstring = "";
variable viewstring = "";
variable hasSubtype = "";
variable hasAction = "";
variable hasCount = 0;
variable filename = "";
variable validAnnotation = 0;
variable specialMark = 0;
variable insertionPage = 0;
variable hasDestination = "";
variable needDestination = 0;
variable isNamedDestination = 0;
variable isOutline = 0;

% Utility function to produce output for the log file
define printf (n)
{
   logprint (Sprintf (n));
}

% Replace all occurances of a string in another string
define str_replace_all (orig, match, replacement)
{
	while (str_replace (orig, match, replacement))
		orig = ();
	return orig;
}


% further utilty functions are

% void objToAnnotation(int pagenumber, string annotation);
% void objToOutline(annotstring);
% void objToName(annotstring);

% addToContent(stringofpdfcommands)



% actions are called by PDF engine whenever a special event happens

% action_pdfmark - collects and filters PDF marks (annotations)
% called whenever a pdfmark dicionary entity is worked on
% s is a string and empty on end of dictionary
% n is a integer counter

define action_pdfmark(s,n) {

	% the string will be zero length at the end of the annoation, so process the collected data
	if (strlen(s) == 0) {
	
		% ****
		% validAnnotation = 0;		% uncomment this line for disabling annotations
		% ****

		if (specialMark > 0) {
			addDocFX(annotstring);
		}

		if (validAnnotation > 0) {
		
		
			% We need to construct a destination for the link
			% #PAGEANNOT=### is a placeholder and will be replaced by dumpPDFBase() by the
			% object number for the ### page object at dumpTime, we take the rest from /View
			variable destination;

			% if we do not yet have seen a subtype we default to /Text (a note)
			if ((strlen(hasSubtype) == 0) and (isOutline == 0) and (isNamedDestination == 0)) {		
				annotstring = strcat(annotstring,"\n/Subtype /Text");
				needDestination++;	% Text annots need to have a /Dest(ination) page
			}

			% if we need a Destination (and there is no need for Actions), add it to the annotation
			if ((needDestination > 0) and (strlen(hasAction) == 0) and (strlen(filename) == 0)) { 
				if (strlen(viewstring) > 0) {   % defining a view, if so - use it!
					destination = Sprintf("\n/Dest [ #PAGEANNOT=%d %s", insertionPage, viewstring ,2 );
				} else {
					destination = Sprintf("\n/Dest [ #PAGEANNOT=%d ]", insertionPage ,1 );
				}
			} else {
				destination = "";	% no destination needed
			}
			
			% add the destination to the annotation
			annotstring = strcat(annotstring,destination);

			% Switch the cases, first prioriy is 'named destionations' as these go into the /Dest array
			% of the document
		
			if (isNamedDestination > 0) {
				if (strlen(viewstring) > 0) {	% do we define a view, if so - use it!
					annotstring = Sprintf("%s [ #PAGEANNOT=%d %s", hasDestination, insertionPage, viewstring ,3 );
				} else {
					annotstring = Sprintf("%s [ #PAGEANNOT=%d ]", hasDestination, insertionPage ,2 );
				}
				objToName(annotstring);	% add the annoation to the document named destinations
			} else {	% otherwise
				if (isOutline > 0) {	% check if the annotation defines an outline 
					if (hasCount != 0) {
						objToOutlineCount(annotstring,hasCount);
					} else {
						objToOutline(annotstring);	% if yes, dump to document outline list
					}
				} else {
					objToAnnotation(curPageNum(),annotstring ); % otherwise the annotation belongs to the current page
				}
			}
		}
		% reset to initial state
		annotstring = "";
		viewstring = "";
		hasSubtype = "";
		hasAction = "";
		hasCount = 0;
		hasDestination = "";
		needDestination = 0;
		filename = "";
		validAnnotation = 0;
		isOutline = 0;
		isNamedDestination = 0;
		specialMark = 0;
		return;
	}

	% is it the /Page key, we need to remember the target page, then drop key
	if (strncmp(s,"/Page",5) == 0) {
		logprint(substr(s, 6, -1));
		if (strncmp(substr(s, 7, -1),"/Next",5) == 0) {
			insertionPage = curPageNum()+1;
		} else {
			insertionPage = integer( substr(s, 6, -1) );
		}
		needDestination++;
		if (insertionPage > 0) return;
	}

	% is it the /View key, we need to remember the target location, then drop key
	if (strncmp(s,"/View",5) == 0) {
		viewstring = substr(s, 8, -1);
		return;
	}

	if (strncmp(s,"/Subtype",8) == 0) {
		% define which kind of subtype needs a destination
		if (string_match(s,"/Link",1)) {
			needDestination++;
		}
		if (string_match(s,"/Text",1)) {
			needDestination++;
		}
		hasSubtype = s;
	}

	if (strncmp(s,"/Dest",5) == 0) {
		hasDestination = substr(s, 7, -1);
	}

	if (strncmp(s,"/Count",6) == 0) {
		hasCount = integer( substr(s, 8, -1) );
	}

	if (strncmp(s,"/Color",6) == 0) {
		% change /Color to /C
		s = Sprintf("/C %s",substr(s, 8, -1),1);
	}

	if (strncmp(s,"/File",5) == 0) {
		% remember a file for later action
		filename = substr(s, 7, -1);
		return;
	}

	% Action processing is tricky, we need to see if the /Action name already has its dict with it
	if (strncmp(s,"/Action",7) == 0) {
		if (strncmp(s,"/Action /",9) == 0) {	% no dict, must be subtype
			s = Sprintf("/A << /S %s ",substr(s, 9, -1),1);  % build the subtype, named /S
			if (strlen(filename) > 0) s = strcat(s, Sprintf("/F %s ",filename,1)); % add a file if any
			s = strcat(s,">>"); % close the dict
		} else {
			s = Sprintf("/A %s",substr(s, 9, -1),1);
			s = str_replace_all (s, "/Subtype", "/S");	% sanitize Subtype
			hasAction = s;
		}
	}

	% be sure to set a newline before adding anything
	if (strlen(annotstring) > 0) {
		annotstring = strcat(annotstring,"\n");
	} else {
		% set the current page as default target page
		insertionPage = curPageNum();

		% we process only old style LNK at the moment
		if (strcmp(s,"/OUT") == 0) {
			validAnnotation++;
			s = "";
			isOutline++;
		}
		if (strcmp(s,"/DEST") == 0) {
			validAnnotation++;
			isNamedDestination++;
			s = "";
		}
		if (strcmp(s,"/LNK") == 0) {
			validAnnotation++;
			s = "/Type /Annot\n/Subtype /Link";
		}
		% we process also ANNs
		if (strcmp(s,"/ANN") == 0) {
			validAnnotation++;
			s = "/Type /Annot";
			printf("Got ANN\n",0);
		}
		
		if (strcmp(s,"/DOCVIEW") == 0) {
			specialMark++;
		}

	}

	% add to annotation, then return for next key/value pair string
	annotstring = strcat(annotstring,s);
	
}

% action_pagestart - called on start of each page description
define action_pagestart() {
	% addToContent(" % Start me up\n");	% Add some PDF commands at the start of the page content
	% of course you can easily add some annotations also automatically at this point
}

% action_pageend - called on end of each page description
define action_pageend() {
	% addToContent(" % End me down\n");	% Add some PDF commands at the end of the page content
	% of course you can easily add some annotations also automatically at this point
}


% END OF CONTROL SCRIPT
